import { Component, OnInit, ViewChild } from '@angular/core';
import { AccountComponent } from '../account/account.component';
import { Account } from '../data-objects';
import { ExtApiService } from '../ext-data-object-service';
import { MessageService } from '../message.service';

@Component({
  selector: 'app-ledger',
  templateUrl: './ledger.component.html',
  styleUrls: ['./ledger.component.css']
})
export class LedgerComponent implements OnInit {

  fallbackAccount: Account
  fallbackAccountId: number


  @ViewChild('fallbackAccountComponent') fallbackAccountComponent: AccountComponent
  

  constructor(
    private extApiService: ExtApiService,
    private messageService: MessageService
  ) { }

  async getAccount(): Promise<void> {
    try {
      this.messageService.add("Trying to load fallback account")
      this.fallbackAccount = await this.extApiService.getAccountByDescription('fallback_account')
      this.messageService.add("Account loaded")
    } catch (err) {
      this.messageService.add(JSON.stringify(err, undefined, 4))
    }
  }

  async ngOnInit(): Promise<void> {
    await this.getAccount()
    this.fallbackAccountId = this.fallbackAccount.id
  }

}
